#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/device.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <linux/cdev.h>
#include <linux/slab.h>
#include <asm/uaccess.h>
#include <linux/usb.h>
#include <linux/string.h>

#include <linux/uaccess.h>
#include <linux/compat.h>

#include "rds-01-public.h"

static const struct usb_device_id rds_01_table[]=
{
  {USB_DEVICE(RDS01_VID, RDS01_PID)},
  { /* Terminating entry */ }
};

MODULE_DEVICE_TABLE(usb, rds_01_table);

static int rds01_open(struct inode *inode, struct file *filp)
{
  return priv_fops_open(inode,filp);
}

static int rds01_release(struct inode *inode, struct file *filp)
{
  return priv_fops_release(inode,filp);
}

static long rds01_ioctl(struct file *filp,unsigned int cmd,unsigned long arg)
{
  return priv_fops_ioctl(filp,cmd,arg);
}

static ssize_t rds01_read(struct file *filp, char __user *buf, size_t count, loff_t *f_pos)
{
  return priv_fops_read(filp,buf,count,f_pos);
}

static ssize_t rds01_write(struct file *filp, const char __user *buf,
                            size_t count, loff_t *f_pos)
{
  return priv_fops_write(filp,buf,count,f_pos);
}
                        
loff_t rds01_lseek(struct file *filp, loff_t Offset,int whence)
{
  return priv_fops_lseek(filp,Offset,whence);
}

#ifdef CONFIG_COMPAT
static long
rds01_compat_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
  return rds01_ioctl(filp, cmd, (unsigned long)compat_ptr(arg));
}
#else
#define rds01_compat_ioctl NULL
#endif /* CONFIG_COMPAT */


static const struct file_operations rds01_fops =
{
  .owner            = THIS_MODULE,
  .unlocked_ioctl   = rds01_ioctl,
  .compat_ioctl     = rds01_compat_ioctl,
  .open             = rds01_open,
  .release          = rds01_release,
  .read             = rds01_read,
  .write            = rds01_write,
  .llseek           = rds01_lseek,
};


static int rds_01_probe(struct usb_interface *interface,
                        const struct usb_device_id *id)
{
  int NewMinor,ret;
  struct cdev *pCharDev;
  if((NewMinor=priv_drv_get_minor())==-1)
  {
    printk(KERN_ERR "rds-01: to many readers.\n");
    return -1;
  }
  pCharDev=(struct cdev*)kzalloc(sizeof(struct cdev),GFP_KERNEL);
  cdev_init(pCharDev,&rds01_fops);
  if((ret=cdev_add(pCharDev,MKDEV(TOUCHM_MAJOR,TOUCHM_MINOR_BASE+NewMinor),1))!=0)
  {
    printk(KERN_ERR "rds-01: add failed %d\n",ret);
    return -ENOMEM;
  }
  return priv_dev_probe(interface,pCharDev,NewMinor);
}

static void rds_01_disconnect(struct usb_interface *interface)
{
  priv_dev_disconnect(interface);
}

static struct usb_driver rds01_driver =
{
  .name         =   "rds-01",
  .probe        =   rds_01_probe,
  .disconnect   =   rds_01_disconnect,
  .id_table     =   rds_01_table,
  .supports_autosuspend = 0,
};

static int __init touchm_init(void)
{
  int res;
  if((res=priv_drv_init())!=0)
    return res;
  usb_register(&rds01_driver);
  return 0;
}

static void __exit touchm_exit(void)
{
  usb_deregister(&rds01_driver);
  priv_drv_close();
}

MODULE_AUTHOR("Pawel E Mironov <pawel0xff@yandex.ru");
MODULE_DESCRIPTION("touchmem driver");
MODULE_LICENSE("GPL");


module_init(touchm_init);
module_exit(touchm_exit);
